<?php

App::uses('CakeEventListener', 'Event');
App::uses('DiscountLib', 'EvDiscount.Lib');

class EvShopCheckForDiscountRestrictionsListener implements CakeEventListener {

	public function implementedEvents() {
		return array(
			'EvBasket.Component.BasketItem.CheckDiscountRestrictions' => 'checkForDiscountRestrictions',
		);
	}

	/**
	 * rebuild discount due to basket amends
	 *
	 * @param 	CakeEvent
	 */
	public function checkForDiscountRestrictions(CakeEvent $Event) {
		$result = array();

		if (
			isset($Event->data['discount']['code_restrictions'])
			&& ! empty($Event->data['discount']['code_restrictions'])
			&& isset($Event->data['basket_item'])
			&& ! empty($Event->data['basket_item'])
		) {

			// Restrictions were found. Check to see if they are for the shop
			$basketItem = $Event->data['basket_item'];
			$discount = $Event->data['discount'];

			// Get the product id for basket item
			if ($basketItem['model'] == 'EvShop.Variant') {
				if (isset($basketItem['Variant']['Product'])) {
					$productId = $basketItem['Variant']['Product']['id'];
				}
			} elseif ($basketItem['model'] == 'EvShop.Product') {
				$productId = $basketItem['model_id'];
			}

			// Default to invalidating the discount
			$result = array(
				'discountValue' => false
			);

			// We've now got a list of product ids in the basket. Now we need to go through each code restriction
			// and work out if it applies to any of the products in the basket.
			foreach ($discount['code_restrictions'] as $restriction) {
				if ($restriction['model'] == 'EvShop.Product') {
					if ($productId === $restriction['model_id']) {
						// It's a match - we allow the product to be discounted.
						$result = array(
							'discountValue' => true
						);
					}
				} elseif ($restriction['model'] == 'EvShop.Brand') {
					// Check if the brand applies to any of the products
					if (
						! empty($basketItem['Variant']['Product']['brand_id']) &&
						! empty($restriction['model_id'])
					) {
						$productBrandId = $basketItem['Variant']['Product']['brand_id'];

						// when a brand discount has been requested, fetch the brand id and
						// all child brand ids associated with it
						$Brand = EvClassRegistry::init('EvShop.Brand');

						$allowedChildBrandIds = $Brand->find('list', array(
							'conditions' => array(
								$Brand->alias . '.parent_id' => $restriction['model_id'],
								$Brand->alias . '.is_active' => true
							),
						));

						if (
							$productBrandId == $restriction['model_id'] ||
							isset($allowedChildBrandIds[$productBrandId])
						) {
							// It's a match - we allow the product to be discounted.
							$result = array(
								'discountValue' => true
							);
						}
					}
				} elseif ($restriction['model'] == 'EvShop.Category') {

					$Category = EvClassRegistry::init('EvShop.Category');
					$parentCategory = $Category->findById($restriction['model_id']);

					// Get the list of categories the product is directly associated with that are under the parent category (includes parent)
					$matchingCategories = $Category->CategoriesProduct->find('list', [
						'fields' => [
							'CategoriesProduct.category_id',
							'CategoriesProduct.category_id',
						],
						'conditions' => [
							'CategoriesProduct.product_id' => $basketItem['Variant']['Product']['id']
						],
						'joins' => [
							[
								'table' => 'ev_shop_categories',
								'alias' => 'Category',
								'type' => 'Inner',
								'conditions' => [
									'Category.id = CategoriesProduct.category_id',
									'Category.lft >= ' => $parentCategory['Category']['lft'],
									'Category.rght <= ' => $parentCategory['Category']['rght'],
								]
							]
						]
					]);

					// If any categories exist then this product should be discount
					if (!empty($matchingCategories)) {
						$result = array(
							'discountValue' => true
						);
					}
				}
			}

			return $result;
		}
		return array();
	}
}
